use [AllNasdaqTickerPricesfrom2014into2017]
go

create procedure [dbo].[generate_buy_sell_prices_for_reversals_by_symbol]
as
begin
	-- SET NOCOUNT ON added to prevent extra result sets from
	-- interfering with SELECT statements.
	SET NOCOUNT ON;

-- create and populate table (#test) for percent_change 
-- from first leave oversold in reversals

begin try
drop table #test
end try
begin catch
print '#test not available to drop'
end catch

-- percent_change within partition
select 
 non_zero_row_numbers.row_number
,non_zero_row_numbers.symbol symbol
,non_zero_row_numbers.date [date]
,non_zero_row_numbers.rsi
,non_zero_row_numbers.[close] row_number_close
,zero_row_numbers.[close] partition_close
,
(
(
cast(non_zero_row_numbers.[close] as real)
/
cast(zero_row_numbers.[close] as real)
)
-1)*100 percent_change
into #test
from
(
-- non-zero row_number records by paritition_rn
select 
--*
 symbol
,date
,rsi
,[close]
,[row_number] [row_number]
,row_number() over(partition by row_number order by date) partition_rn
from ##rsi_close_history 
where row_number != 0
) non_zero_row_numbers

left join

(
-- zero row_number records by paritition_rn
select 
symbol
,date
,rsi
,[close]
,row_number
,row_number() over(partition by row_number order by date) partition_rn
from ##rsi_close_history 
where row_number = 0
) zero_row_numbers

on non_zero_row_numbers.row_number = zero_row_numbers.partition_rn

--where non_zero_row_numbers.partition_rn = 1 and zero_row_numbers.partition_rn = 1


order by non_zero_row_numbers.date



-- populate #test2 based on #test
-- and logic described below

-- assign sells for 1a and 1b rules
 
-- rule 1a assigns 'sell' to sell column
-- when percent_change less than -20 
-- for the first leaves overbought date

-- rule 1b assigns 'sell' to sell column
-- when there is just a sole leaves overbought
-- record to a reversal cycle as 
-- denoted by a row_number value
-- if rules 1a and 1b do not apply 
-- assign null to sell column

-- in addition to sell column add a 
-- max_percent_change column and 
-- change_from_max_percent_change
-- max_percent_change is maximum percent_change
-- value with the same row_number for a symbol
-- change_from_max_percent change is
-- percent_change less - max_percent_change

-- to #test2 table and add max_percent_change
-- and change_from_max_percent_change
-- also add and populate sell rows to sell for
-- for change_from_max_percent_change decline of 
-- more than 20 percent or
-- only one leaves overbought record in a reversal

begin try
drop table #test2
end try
begin catch
print '#test2 not available to drop'
end catch


-- read temp table with percent_change  
select 
 row_number
,symbol
,[date]
,row_number_close
,partition_close
,percent_change
,max(percent_change) over(partition by row_number order by date) max_percent_change
,
percent_change
-
max(percent_change) over(partition by row_number order by date) change_from_max_percent_change
,
case
	when percent_change < -20 and date = min(date) over (partition by row_number order by date) then 'sell' 
	when 
		lag(row_number,1) over (partition by row_number order by date) is null
		and lead(row_number,1) over (partition by row_number order by date) is null
		then 'sell'
	else null
end sell

into #test2

from #test
order by row_number, date



----------------------------------------------------------------------------------------

-- process #test2 rows in sets of multiple 
-- leaves overbought records for a symbol 
-- so that 'sell' is assigned to sell column for 
-- first row with percent_change less max_percent_change
-- having a value more negative than -20 

-- this is label for goto loop
update_for_change_from_max_percent_change_lt_minus_20_criterion:
-- date for target sell to update
declare @date_for_sell_b4_update date =
(select date from #test2
where 
date =
(
-- row_number with min_date for change_from_max_percent_change < -20
select top 1 min(date) over (partition by row_number order by date)
from #test2
where 
row_number in 
(
select distinct row_number from #test2 
where row_number not in (select distinct row_number from #test2 where sell is not null)
) 
and sell is null 
and change_from_max_percent_change < -20)
)


-- row_number for target sell to update
declare @row_number_for_sell_b4_update bigint =
(select row_number from #test2
where 
date =
(
-- row_number with min_date for change_from_max_percent_change < -20
select top 1 min(date) over (partition by row_number order by date)
from #test2
where 
row_number in 
(
select distinct row_number from #test2 
where row_number not in (select distinct row_number from #test2 where sell is not null)
) 
and sell is null 
and change_from_max_percent_change < -20)
)


-- perform update
update #test2
set sell = 'sell'
where 
date = @date_for_sell_b4_update
and row_number = @row_number_for_sell_b4_update



-- loop back to instruction with 
-- update_for_change_from_max_percent_change_lt_minus_20_criterion label
-- if there are additional records to process
if
(select count(*)
from #test2
where 
row_number in (select distinct row_number from #test2) 
and row_number != @row_number_for_sell_b4_update 
and sell is null 
and change_from_max_percent_change < -20
) = 0
begin
print 'no more to change for change_from_max_percent_change < -20 criterion'
end
else
begin
print 'there are targets remaining to be changed for change_from_max_percent_change < -20 criterion'
goto update_for_change_from_max_percent_change_lt_minus_20_criterion;
end

--*/
-----------------------------------------------------------------------------------------------------

-- this last section is for all #test2 rows that are not processed
-- by a prior rule to designate a 'sell' row in the sell column

-- perform update for 60% of max close
-- on normal exit from leave overbought with no more
update #test2

set row_number_close = (max_close - partition_close)*.6 + partition_close,
	sell = 'sell',
	percent_change = 0,
	change_from_max_percent_change = 0


-- from clause for join to #test2
from

(
-- get max_close rows
select 
 row_number
,symbol
,max(row_number_close) max_close
,max(date) last_date
from #test2
where row_number in
(
-- row_number values without a sell
select distinct row_number from #test2
where sell is null

except

select row_number from #test2
where sell = 'sell'
)
group by 
 row_number
,symbol
) for_nu_row_number_close
inner join #test2
on #test2.symbol = for_nu_row_number_close.symbol
and #test2.date = for_nu_row_number_close.last_date
and #test2.row_number = for_nu_row_number_close.row_number



-- insert sell rows from #test2
-- into ##sell_rows_from_#test2

-- row_number_close = sell price
-- partition_close = buy price

insert into ##sell_rows_from_#test2
select *
from #test2 where sell = 'sell'
end

go


