use [AllNasdaqTickerPricesfrom2014into2017]
go

create procedure [dbo].[generate_rsi_close_history_for_symbol]  @symbol_in varchar(5)

AS
BEGIN
	set nocount on;

declare 
 @symbol varchar(5) = @symbol_in
,@max_row_number smallint
,@current_row_number smallint 




-- #starting_ending_leaves_overbought_dates has one
-- row for each reversal cycle for a symbol
-- start_date and end_date are for date of
-- first overbought record in each reversal
-- row_number denotes reversal cycle number
-- source data is from ##rsi_reversal_trades
begin try
drop table #starting_ending_leaves_overbought_dates
end try
begin catch
print '#starting_ending_leaves_overbought_dates table not available to drop'
end catch



select 
 starting_leaves_overbought.symbol
,starting_leaves_overbought.date [start_date]
,starting_leaves_overbought.rsi
,starting_leaves_overbought.[close] [close]
,ending_leaves_overbought.date end_date
,ending_leaves_overbought.row_number row_number
into #starting_ending_leaves_overbought_dates
from 
(
-- get all starting leaves overbought
-- get all leaves overbought
-- immediately after leaves oversold
select *, row_number() over (order by date) row_number
from
(
-- get all records for @symbol
-- along with lag and lead source values
select 
 symbol
,date
,rsi
,[close] [close]
,[source] [source]
,lag([source],1) over (order by date) source_lag_1
,lead([source],1) over (order by date) source_lead_1 
from ##rsi_reversal_trades
where 
symbol = @symbol 
) for_first_leaves_overbought
where 
[source] = 'leaves overbought'
and source_lag_1 = 'leaves oversold'
) starting_leaves_overbought


inner join

(
-- get all ending leaves overbought
-- get all leaves overbought
-- immediately befor leaves oversold
-- or before is null source
select *, row_number() over (order by date) row_number
from
(
-- get all records for @symbol
-- along with lag and lead source values
select 
 symbol
,date
,rsi
,[close] [close]
,[source] [source]
,lag([source],1) over (order by date) source_lag_1
,lead([source],1) over (order by date) source_lead_1 
from ##rsi_reversal_trades
where 
symbol = @symbol 
) for_last_leaves_overbought
where 
[source] = 'leaves overbought'
and 
(
source_lead_1 = 'leaves oversold'
or source_lead_1 is null
)
) ending_leaves_overbought
on
starting_leaves_overbought.row_number = ending_leaves_overbought.row_number



-- populate ##rsi_close_history records from
-- ##rsi_reversal_trades table and 
-- #starting_ending_leaves_overbought_dates table
-- and the rsi table

-- ##rsi_close_history has at least two records
-- for every reversal cycle for a symbol

-- row_number 0 denotes the first leaves oversold 
-- record for a reversal cycle
-- other row_number values indicate the reversal 
-- cycle number from the #starting_ending_leaves_overbought_dates table

-- for reversal cycles with more than one leaves overbought record
-- history of rsi table row values included from beginning
-- leaves oversold record through last leaves overbought record

begin try
drop table ##rsi_close_history 
end try
begin catch
print '##rsi_close_history table not available to drop'
end catch

-- insert  the beginning of all reversals
-- all beginning reversals have source = leaves oversold
-- and source_lag_1 is null or = leaves overbought
select 
symbol
,date
,rsi
,[close] [close]
,0 row_number
into ##rsi_close_history
from
(
-- get all records for @symbol
-- along with lag and lead source values
select 
 symbol
,date
,rsi
,[close] [close]
,[source] [source]
,lag([source],1) over (order by date) source_lag_1
,lead([source],1) over (order by date) source_lead_1 
from ##rsi_reversal_trades
where 
symbol = @symbol 
) for_last_leaves_overbought
where 
[source] = 'leaves oversold'
--/*
and 
(
source_lag_1 = 'leaves overbought'
or source_lag_1 is null
)



-- add rows from rsi table
-- set local variables for current value of @symbol
set @max_row_number = (select max(row_number) from #starting_ending_leaves_overbought_dates)
set @current_row_number = 1


-- insert beginning overbought records
-- and intermediate rsi records if
-- there are overbought records following one another
while @current_row_number <= @max_row_number
begin


if 
(select start_date from #starting_ending_leaves_overbought_dates where row_number = @current_row_number) = 
(select end_date from #starting_ending_leaves_overbought_dates where row_number = @current_row_number) 
begin
insert into ##rsi_close_history
select 
 symbol
,start_date
,rsi
,[close]
,@current_row_number  
from #starting_ending_leaves_overbought_dates  
where row_number = @current_row_number
end
else
begin 
-- insert multiple rows from rsi table
-- based on start_date and end_date from 
-- for current row in
-- #starting_ending_leaves_overbought_dates
insert into ##rsi_close_history
select 
symbol
,date
,rsi
,[close]
,@current_row_number 
from rsi 
where symbol = @symbol 
and date >= (select start_date from #starting_ending_leaves_overbought_dates where row_number = @current_row_number)
and date <= (select end_date from #starting_ending_leaves_overbought_dates where row_number = @current_row_number) 
order by date
end


set @current_row_number = @current_row_number + 1

end

end

go


